# MusicRoom(v6.14)に以下のActionを追加した
# MusicRoom2を使用可能にする

# RandomPlay() ランダムに再生する
# Toggleplay() 再生時は停止、非再生時は再生する
# Repeat() 連続再生をオン/オフするボタン
# Loop() ループ再生をオン/オフするボタン
# ToggleConfig() ループ再生と連続再生をトグルする。ループの方が真
# MRcheck() タイマーをセットする
# 
# また、addに引数timeを追加した。
#例
#    mr = MusicRoom2()
#    mr.playback = "test"
#    mr.add("track1.ogg", always_unlocked=True, time=7 * 60 + 24)
#    mr.add("track2.ogg", always_unlocked=True, time=1 * 60 + 46)
#    mr.add("track3.ogg", always_unlocked=True, time=2 * 60 + 45)

# ミュージックルームのscreenには、必ず以下のタイマーを追加してください。
# timer 1.0 repeat True action mr.MRcheck()


# 秒単位で曲の長さを指定することで以下のメンバ変数を更新するようになる

# 再生位置を文字列で表わすDynamicDisplayableを返す、初期値は"00:00/00:00"
# mr.position_text(**properties)
# 現在の再生位置と全体の長さをミリ秒単位の整数で返す
# mr.pos
# mr.length
# 次のように文字を表示する
# add mr.position_text(size=55, italic=True)

# 曲の長さに対する現在位置の割合(0~1)
# self.bar
# 任意の場所でバーを表示する
# $ pos_bar = ui.adjustment(value=mr.bar)
# bar adjustment pos_bar

# バーを表示する時は次のようにタイマーを変更しなければ表示が更新されない
# timer 1.0 repeat True action mr.MRcheck(pos_bar)

#例
#init -2 python:
#
#
#    mr = MusicRoom2()
#    mr.add("track1.ogg", always_unlocked=True)
#    mr.add("track2.ogg", always_unlocked=True)
#    mr.add("track3.ogg", always_unlocked=True)
#
#screen music_room:
#
#    tag menu
#    timer .1 repeat True action mr.MRcheck()
#
#    frame:
#        has vbox
#
#        # The buttons that play each track.
#        textbutton "Track 1" action mr.Play("track1.ogg")
#        textbutton "Track 2" action mr.Play("track2.ogg")
#        textbutton "Track 3" action mr.Play("track3.ogg")
#
#        null height 20
#
#        # Buttons that let us advance tracks.
#        textbutton "Next" action mr.Next()
#        textbutton "TogglePlay" action mr.TogglePlay()
#        textbutton "RandomPlay" action mr.RandomPlay()
#        textbutton "Previous" action mr.Previous()
#
#        null height 20
#
#        # The button that lets the user exit the music room.
#        textbutton "Main Menu" action ShowMenu("main_menu")
#        textbutton "loop"        action mr.Loop()
#        textbutton "repeat" action mr.Repeat()




init -2 python:

    class __MRLoop(Action):
        def __init__(self, mr):
            self.mr = mr
        def __call__(self):
            self.mr.repeat = False
            if self.mr.loop:
                self.mr.loop = False
            else:
                self.mr.loop = True
        def get_selected(self):
            if self.mr.loop:
                return True

    class __MRRepeat(Action):
        def __init__(self, mr):
            self.mr = mr
        def __call__(self):
            self.mr.loop = False
            if self.mr.repeat:
                self.mr.repeat = False
            else:
                self.mr.repeat = True
        def get_selected(self):
            if self.mr.repeat:
                return True

    class __MRToggleConfig(Action):
        def __init__(self, mr):
            self.mr = mr
            if not self.mr.loop and not self.mr.repeat:
                self.mr.repeat = True
        def __call__(self):
            if self.mr.loop:
                self.mr.loop = False
                self.mr.repeat = True
            else:
                self.mr.loop = True
                self.mr.repeat = False
        def get_selected(self):
            if self.mr.loop:
                return True

    class __MRRandomPlay(Action):
        def __init__(self, mr):
            self.mr = mr
        def __call__(self):
            self.mr.playback = renpy.random.choice(self.mr.unlocked_playlist())
            self.mr.play(self.mr.playback)

    class __MRTogglePlay(Action):
        def __init__(self, mr):
            self.mr = mr

        def __call__(self):
            if renpy.music.get_playing(channel=self.mr.channel) is not None:
                return self.mr.stop()
            return self.mr.play()

        def get_selected(self):
            return renpy.music.get_playing(channel=self.mr.channel) is not None

    class __MRcheck(Action):
        def __init__(self, mr, pos_bar):
            self.mr = mr
            self.pos_bar = pos_bar

        def __call__(self):
            if self.mr.pausing:
                return
            playing = renpy.music.get_playing(channel=self.mr.channel)

            # 再生位置
            if self.mr.lengthes.get(self.mr.playback, None) is not None:
                length = self.mr.lengthes[self.mr.playback] * 1000
                pos = renpy.audio.audio.get_channel("music").get_pos()
                if pos > length or pos < 0:
                    pos = length
                self.mr.bar = float(pos) / float(length)
                if self.pos_bar is not None:
                    self.pos_bar.change(self.mr.bar)
                self.mr.pos = pos
                self.mr.length = length
                self.mr.text = "%02d:%02d/%02d:%02d" % ( pos / 60000, (pos % 60000) / 1000, length / 60000, (length % 60000) / 1000)
            # loop, 連続再生処理
            if (self.mr.repeat or self.mr.loop) and playing is None:

                playlist = self.mr.unlocked_playlist()
                if not playlist:
                    return
                
                try:
                    idx = playlist.index(self.mr.playback)
                except ValueError:
                    idx = 0

                if self.mr.repeat:
                    idx = idx + 1
                    if idx > len(playlist) - 1:
                        idx = 0
                self.mr.playback = playlist[idx]
                self.mr.play(self.mr.playback)

    class MusicRoom2(MusicRoom):
        
        def __init__(self):
            super(MusicRoom2, self).__init__()
    
            self.loop = False
            self.repeat = False
            self.playback = None
            # 再生位置表示用
            self.bar = 0
            self.text = "00:00/00:00"
            # self.start = 0
            self.lengthes = {}
            self.pausing = True
            self.pos = 0
            self.length = 0

        def play(self, filename=None, offset=0):
            # """
            # 勝手に連続再生しないようにする
            # マーカーを追加
            # """
            
            playlist = self.unlocked_playlist()
            
            if not playlist:
                return
            
            if filename is None:
                # filename = renpy.music.get_playing(channel=self.channel)
                filename = self.playback
            
            try:
                idx = playlist.index(filename)
            except ValueError:
                idx = 0
            
            idx = idx + offset
            if idx < 0:
                idx = len(playlist) - 1
            elif idx > len(playlist) - 1:
                idx = 0

            self.playback = playlist[idx]
            self.pausing = False
            renpy.music.play(self.playback, channel=self.channel, fadeout=self.fadeout, fadein=self.fadein, loop=False)

        def stop(self):
            """
            # マーカーを追加
            """
            self.pausing = True
            renpy.music.stop(channel=self.channel, fadeout=self.fadeout)


        def add(self, filename, always_unlocked=False, time=None):
            """
            """
            self.lengthes[filename] = time
            self.playlist.append(filename)
            self.filenames.add(filename)
            
            if always_unlocked:
                self.always_unlocked.add(filename)

        # 追加関数↓
        def Loop(self):
            return __MRLoop(self)
        def Repeat(self):
            return __MRRepeat(self)
        def ToggleConfig(self):
            return __MRToggleConfig(self)
        def RandomPlay(self):
            return __MRRandomPlay(self)
        def MRcheck(self, pos_bar=None):
            return __MRcheck(self, pos_bar)
        def TogglePlay(self):
            return __MRTogglePlay(self)
        def position_text(self, *args, **kwargs):
            return DynamicDisplayable(self.dynamic_text, **kwargs)
        def dynamic_text(self, st, at, **kwargs):
            return Text(self.text, **kwargs), .1
